"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ValidateRequestAsync = exports.ValidateRequest = exports.validateFileExtension = exports.validateImage = exports.validateFile = exports.failValidationForField = exports.validateRequestAsync = exports.validateRequest = exports.resolveValidationError = void 0;
const path_1 = __importDefault(require("path"));
const joi_1 = __importDefault(require("joi"));
const config_1 = require("@men-mvc/config");
const types_1 = require("../types");
const response_1 = require("./response");
const error_1 = require("./error");
const resolveValidationError = (valError) => {
    if (!valError) {
        return new types_1.ValidationError({});
    }
    let errors = {};
    valError.details.map((error) => {
        errors[error.context?.key] = error.message;
    });
    return new types_1.ValidationError(errors);
};
exports.resolveValidationError = resolveValidationError;
const validateRequest = (schema, data, options = {
    abortEarly: false
}) => {
    const valResult = schema.validate(data, options);
    if (valResult.error) {
        throw (0, exports.resolveValidationError)(valResult.error);
    }
};
exports.validateRequest = validateRequest;
const validateRequestAsync = async (schema, data, options = {
    abortEarly: false
}) => {
    /**
     * Joi will throw built-in ValidationError for async validation
     */
    await schema.validateAsync(data, options);
};
exports.validateRequestAsync = validateRequestAsync;
const failValidationForField = (field, errorMessage) => {
    throw new types_1.ValidationError({
        [field]: errorMessage
    });
};
exports.failValidationForField = failValidationForField;
const isUploadedFile = (value) => value instanceof types_1.UploadedFile;
const isImageFile = (file) => {
    let imageMimeTypes = ['image/gif', 'image/jpeg', 'image/png'];
    const additionalImageMimesCsv = (0, config_1.getEnvVariable)('UPLOADED_FILE_IMAGE_MIMES', '');
    if (additionalImageMimesCsv) {
        const additionalMimes = additionalImageMimesCsv
            .split(',')
            .map((mime) => mime.toLowerCase());
        imageMimeTypes = imageMimeTypes.concat(additionalMimes);
    }
    return imageMimeTypes.includes(file.mimetype.toLowerCase());
};
const validateImageFile = (field, value, error) => {
    if (!isUploadedFile(value)) {
        // input value is not a file
        return (0, exports.failValidationForField)(field, error);
    }
    if (!isImageFile(value)) {
        (0, exports.failValidationForField)(field, error);
    }
};
/**
 * This function can be used to validate both multiple-file upload and single upload
 */
const validateFile = (value, field, message) => {
    const error = message ?? `Input value(s) must be file(s).`;
    if (!value) {
        // if the value is empty, it does not validate if the input is file or not
        return;
    }
    if (Array.isArray(value)) {
        const values = value;
        for (let element of values) {
            if (!isUploadedFile(element)) {
                (0, exports.failValidationForField)(field, error);
            }
        }
    }
    else if (!isUploadedFile(value)) {
        // single file upload
        (0, exports.failValidationForField)(field, error);
    }
};
exports.validateFile = validateFile;
/**
 * This function can be used to validate both multiple-file upload and single upload
 */
const validateImage = (value, field, message) => {
    const error = message ?? `Invalid image file(s).`;
    if (!value) {
        // when the input field is empty, it will pass.
        return;
    }
    if (Array.isArray(value)) {
        // multiple files
        value.map((file) => validateImageFile(field, file, error));
    }
    else {
        validateImageFile(field, value, error);
    }
};
exports.validateImage = validateImage;
/**
 * This function can be used to validate both multiple-file upload and single upload
 */
const validateFileExtension = (value, field, allowedExtensions, // including .
message) => {
    if (!value) {
        return;
    }
    if (allowedExtensions.length < 1) {
        return;
    }
    const error = message ?? `File does not have the valid extension.`;
    const validate = (singleVal) => {
        if (!isUploadedFile(singleVal)) {
            return false;
        }
        return allowedExtensions.some((ext) => ext.toLowerCase() ===
            path_1.default.extname(singleVal.originalFilename).toLowerCase());
    };
    if (Array.isArray(value)) {
        for (let file of value) {
            if (!validate(file)) {
                (0, exports.failValidationForField)(field, error);
            }
        }
    }
    else if (!validate(value)) {
        (0, exports.failValidationForField)(field, error);
    }
};
exports.validateFileExtension = validateFileExtension;
function ValidateRequest(schemaOrValidator) {
    return (target, propertyKey, descriptor) => {
        const originalMethod = descriptor.value;
        descriptor.value = function (req, res, next) {
            try {
                (0, exports.validateRequest)((0, types_1.isRequestValidator)(schemaOrValidator)
                    ? schemaOrValidator.getSchema(req, res)
                    : schemaOrValidator, req.body);
            }
            catch (e) {
                if (e instanceof types_1.ValidationError) {
                    return (0, response_1.validationErrorResponse)(res, e);
                }
                return (0, error_1.invokeRequestErrorHandler)(e, req, res);
            }
            return originalMethod.apply(this, [req, res, next]);
        };
        return descriptor;
    };
}
exports.ValidateRequest = ValidateRequest;
const ValidateRequestAsync = (schemaOrValidator) => {
    return function (scope, methodName, descriptor) {
        const originalMethod = descriptor.value;
        descriptor.value = async function (req, res, ...args) {
            try {
                await (0, exports.validateRequestAsync)((0, types_1.isRequestValidator)(schemaOrValidator)
                    ? schemaOrValidator.getSchema(req, res)
                    : schemaOrValidator, req.body);
            }
            catch (e) {
                if (e instanceof types_1.ValidationError) {
                    return (0, response_1.validationErrorResponse)(res, e);
                }
                else if (e instanceof joi_1.default.ValidationError) {
                    return (0, response_1.validationErrorResponse)(res, (0, exports.resolveValidationError)(e));
                }
                return (0, error_1.invokeRequestErrorHandler)(e, req, res);
            }
            return originalMethod.apply(this, [req, res, ...args]);
        };
    };
};
exports.ValidateRequestAsync = ValidateRequestAsync;
